/*
 * This file is part of the Kvkbd project.
 * Copyright (C) 2007-2008 Todor Gyumyushev <yodor@developer.bg>
 * Copyright (C) 2008 Guillaume Martres <smarter@ubuntu.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "resizabledragwidget.h"

#include <QDebug>
#include <QPoint>
#include <QPainter>
#include <QMouseEvent>

ResizableDragWidget::ResizableDragWidget(QWidget *parent, Qt::WindowFlags f)
	: DragWidget(parent, f)
{
	if (!(f | Qt::X11BypassWindowManagerHint)) setLocked(true); //no resize in strut mode

	toResize = false;
}


ResizableDragWidget::~ResizableDragWidget()
{
}

void ResizableDragWidget::mousePressEvent(QMouseEvent * e)
{
	if (isLocked())
		return;

	QPoint pos = e->pos();
	if (pos.x() > width() - 20 && pos.x() < width() && pos.y() > height() - 20 && pos.y() < height())
	{
		toResize = true;
		dragPoint = QPoint(width() - pos.x(), height() - pos.y());
		resizeType = ResizeBottomRight;
	}
	else if (pos.x() < 20 && pos.y() < 20)
	{
		toResize = true;
		dragPoint = pos;
		resizeType = ResizeTopLeft;
	}
	else if (pos.x() > width() - 20 && pos.x() < width() && pos.y() < 20)
	{
		toResize = true;
		dragPoint = QPoint(width() - pos.x(), pos.y());
		resizeType = ResizeTopRight;
	}
	else if (pos.x() < 20 && pos.y() > height() - 20 && pos.y() < height())
	{
		toResize = true;
		dragPoint = QPoint(pos.x(), height() - pos.y());
		resizeType = ResizeBottomLeft;
	}
	else
	{
		DragWidget::mousePressEvent(e);
	}
}

void ResizableDragWidget::mouseMoveEvent(QMouseEvent * e)
{
	if (!toResize) {
		DragWidget::mouseMoveEvent(e);
		return;
	}

	if (resizeType == ResizeBottomRight)
	{
		setCursor(Qt::SizeFDiagCursor);
		QPoint curr = e->globalPos();
		QPoint pos = QWidget::pos();
		int nw = curr.x() - pos.x() + dragPoint.x();
		int nh = curr.y() - pos.y() + dragPoint.y();

		resize(nw, nh);
	}
	else if (resizeType == ResizeTopLeft)
	{
		setCursor(Qt::SizeFDiagCursor);
		QPoint curr = e->globalPos();
		QPoint pos = geometry().bottomRight() + QPoint(1, 1);

		int nw =  qMax(pos.x() - curr.x() + dragPoint.x(), minimumWidth());
		int nh =  qMax(pos.y() - curr.y() + dragPoint.y(), minimumHeight());

		int x = pos.x() - nw;
		int y = pos.y() - nh;

		setGeometry(x, y, nw, nh);
	}
	else if (resizeType == ResizeTopRight)
	{
		setCursor(Qt::SizeBDiagCursor);
		QPoint curr = e->globalPos();
		QPoint pos = geometry().bottomLeft() + QPoint(0, 1);

		int nw =  qMax(curr.x() - pos.x() + dragPoint.x(), minimumWidth());
		int nh =  qMax(pos.y() - curr.y() + dragPoint.y(), minimumHeight());

		int x = this->x();
		int y = pos.y() - nh;

		setGeometry(x, y, nw, nh);
	}
	else if (resizeType == ResizeBottomLeft)
	{
		setCursor(Qt::SizeBDiagCursor);
		QPoint curr = e->globalPos();
		QPoint pos = geometry().topRight() + QPoint(1, 0);

		int nw =  qMax(pos.x() - curr.x() + dragPoint.x(), minimumWidth());
		int nh =  qMax(curr.y() - pos.y() + dragPoint.y(), minimumHeight());

		int x = pos.x() - nw;
		int y = this->y();

		setGeometry(x, y, nw, nh);
	}
}

void ResizableDragWidget::mouseReleaseEvent(QMouseEvent * e)
{
	if (!toResize) {
		DragWidget::mouseReleaseEvent(e);
		return;
	}
	unsetCursor();
	toResize = false;
}

void ResizableDragWidget::paintEvent(QPaintEvent *)
{
	if (isLocked()) return; //alex

	QPainter p(this);

	for (int a = 0; a < 20; a += 5)
	{
		p.setPen(QColor(170, 0, 0));
		p.drawLine(width() - 20 + a, height() - 2, width() - 2, height() - 20 + a);
		p.setPen(QColor(200, 0, 0));
		p.drawLine(width() - 19 + a, height() - 2, width() - 2, height() - 19 + a);
	}

	for (int a = 0; a < 20; a += 5)
	{
		p.setPen(QColor(170, 0, 0));
		p.drawLine(20 - a, 2, 2, 20 - a);
		p.setPen(QColor(200, 0, 0));
		p.drawLine(19 - a, 2, 2, 19 - a);
	}

	for (int a = 0; a < 20; a += 5)
	{
		p.setPen(QColor(170, 0, 0));
		p.drawLine(width() - 20 + a, 2, width() - 2, 20 - a);
		p.setPen(QColor(200, 0, 0));
		p.drawLine(width() - 19 + a, 2, width() - 2, 19 - a);
	}

	for (int a = 0; a < 20; a += 5)
	{
		p.setPen(QColor(170, 0, 0));
		p.drawLine(20 - a, height() - 2, 2, height() - 20 + a);
		p.setPen(QColor(200, 0, 0));
		p.drawLine(19 - a, height() - 2, 2, height() - 19 + a);
	}
}
