/*
 * This file is part of the Kvkbd project.
 * Copyright (C) 2007-2008 Todor Gyumyushev <yodor@developer.bg>
 * Copyright (C) 2008 Guillaume Martres <smarter@ubuntu.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "vbutton.h"
#include "main.h"

#include <QApplication>
#include <QSequentialAnimationGroup>
#include <QPropertyAnimation>

#include <QMouseEvent>
#include <QTimer>
#include <QDebug>
#include <QX11Info>

#include <math.h>

#include <X11/XKBlib.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>
#include <X11/Xutil.h>
#include "fixx11h.h"

double VButton::pw = 535;//495;
double VButton::ph = 170;

extern VButton *globalShift, *rightShift;
extern QString *currentString;

extern bool autoRepeated, processRightButton;
extern int  vkbdAutoRepeatDelay, vkbdAutoRepeatRate;

static VButton *timerActiveBtn=nullptr;

VButton::VButton(QWidget *parent): QPushButton(parent)
{
	setFocusPolicy(Qt::NoFocus);

	resize(33, 30);
	rightClicked = pressed = grabbed = false;
	keyTimer = new QTimer(this);
	timerStartPos = QPoint(-1,-1);

	animGroup=NULL;

	connect(keyTimer, SIGNAL(timeout()), this, SLOT(repeatKey()));
}
VButton::~VButton()
{
	if (animGroup) delete animGroup;
}

void VButton::setKeyCap(bool shiftPressed, bool capsPressed)
{
	qDebug() << Q_FUNC_INFO;
	//TODO: find a better way to detect if the key is an alphabetic character
	bool capsKey = (text != text.toUpper());
	QString newText;

	if (capsPressed && capsKey) 
		 newText = (shiftPressed ? shiftText : text.toUpper());
	else newText = (shiftPressed ? shiftText : text);

	QPushButton::setText(newText);
}

void VButton::setText(const QString& input, const QString& shiftInput)
{
	text = input;

	QPushButton::setText(text);

	if (!shiftInput.isNull()) 	 shiftText = shiftInput;
	else if (shiftText.isNull()) shiftText = text.toUpper();
}
void VButton::setShiftText(const QString& input)
{
	shiftText = input;
}

void VButton::setKeyCode(unsigned int keycode)
{
	this->keycode = keycode;

}
unsigned int VButton::getKeyCode()
{
	return this->keycode;
}

void VButton::sendKey()
{
#ifdef WITH_SONNET
  if (currentString)
  {
        QString s = (globalShift->isChecked() || rightShift->isChecked())? shiftText : text;

        //fprintf(stderr,"press='%s'\n",s.toLocal8Bit().data());

        if (s.length()==1 && s[0].isLetter())
        {
            currentString->append(s);
        }
        else
        {
            KeySym keysym = XkbKeycodeToKeysym(QX11Info::display(), keycode, 0, 0);
            if (keysym != XK_Shift_L && keysym != XK_Shift_R)
            {
               if (keysym == XK_BackSpace) //what about XK_Delete?
                    currentString->chop(1);
               else currentString->clear();
            }
        }
        //fprintf(stderr,"current string='%s'\n",currentString->toLocal8Bit().data());
  }
#endif
  emit keyClick(keycode);
}

void VButton::reposition(int width, int height)
{
	double dx  = pw / originalSize.x();
	double dy  = ph / originalSize.y();
	double sdx = pw / originalSize.width();
	double sdy = ph / originalSize.height();

	setGeometry(round(width / dx), round(height / dy), round(width / sdx), round(height / sdy));
	//move(round(width / dx), round(height / dy));
	//resize(round(width / sdx), round(height / sdy));

}
void VButton::res()
{
	originalSize = geometry();
}

void VButton::startTimer()
{
	stopTimer();
	keyTimer->start(vkbdAutoRepeatDelay);
	timerActiveBtn=this;
}

void VButton::stopTimer()
{
	if (timerActiveBtn && timerActiveBtn != this) {
		timerActiveBtn->keyTimer->stop();
		//timerActiveBtn->setDown(false);
	}
	keyTimer->stop();
	timerActiveBtn=nullptr;
	//setDown(false);
}

//When a button is pressed, a timer is set which regularly check if
//the button is still pressed and repeat the key if it's the case
void VButton::mousePressEvent(QMouseEvent *e)
{
	if (!grabbed) {
		grabMouse();
		grabbed=true;
	}
	//fprintf(stderr,"mpress %d, btn=%d grabbed=%d\n",keycode,e->button(),grabbed);

    if (e->button()==Qt::RightButton && e->source()!=Qt::MouseEventNotSynthesized) return;

	rightClicked = false;
	if (processRightButton && e->button() == Qt::RightButton) {
		if (globalShift) { //FIXME: is this check really necessary?
			rightClicked = true;
			qDebug() << Q_FUNC_INFO << "checking global shift";
			globalShift->setChecked(true);
		}
		else
			qDebug() << Q_FUNC_INFO << "no global shift";
	}
	if (!keyTimer->isActive())
	{
		//200 ms is a bit more than the time needed for a single click
		//if (e->spontaneous()) //alex: dont start timer (to repeat keys) if event was sent from xlock
		timerStartPos = e->globalPos();

		startTimer();

		if (!keyOnButtonRelease) {
			//alex: Firefox can do ugly mouse grab in its address line and
			//      no mouse release event comes after press but again press-release
			if (!autoRepeated)
			{
			  if (!pressed) sendKey();
			}
			else sendKey();
		}
	}
	else stopTimer();

	QPushButton::mousePressEvent(e);
	pressed=true;

	if (e->button()==Qt::MidButton || e->button()==Qt::RightButton) QPushButton::setDown(true);

	raise();
	QCoreApplication::sendPostedEvents();

	if (!animGroup) 
	{
		animGroup = new QSequentialAnimationGroup();
	}
	else if (animGroup->state()!=QAbstractAnimation::Stopped) //stop and restore
	{
		animGroup->stop();
		QPropertyAnimation *anim1 = (QPropertyAnimation *)animGroup->animationAt(0); 
		QRect rect1 = anim1->startValue().toRect();
		setGeometry(rect1);
	}
	animGroup->clear();

	QRect rect1 = geometry();
	QRect rect2 = QRect(rect1.x()     -rect1.width()/4, 
						rect1.y()     -rect1.height()/2,
						rect1.width() +rect1.width()/2, 
						rect1.height()+rect1.height()/2);

	QPropertyAnimation *anim1 = new QPropertyAnimation(this, "geometry");
	anim1->setDuration(100);
	anim1->setStartValue(rect1);
	anim1->setEndValue(rect2);
	animGroup->addAnimation(anim1);

	QPropertyAnimation *anim2 = new QPropertyAnimation(this, "geometry");
	anim2->setDuration(50);
	anim2->setStartValue(rect2);
	anim2->setEndValue(rect1);
	animGroup->addAnimation(anim2);

	animGroup->start();
}


void VButton::mouseReleaseEvent(QMouseEvent *e)
{
	//fprintf(stderr,"release %d, btn=%d, grabbed=%d\n",keycode,e->button(),grabbed);

	stopTimer();
    if (e->button()==Qt::RightButton && e->source()!=Qt::MouseEventNotSynthesized) return;

	QPushButton::mouseReleaseEvent(e);
	pressed=false;
	if (e->button()==Qt::MidButton || e->button()==Qt::RightButton) QPushButton::setDown(false);

	if (keyOnButtonRelease) sendKey();

	if (grabbed) {
		grabbed=false;
		releaseMouse();
	}
}

void VButton::leaveEvent( QEvent * e) //alex: stop repeat if outside of button (useful on touch screen)
{
	stopTimer();
	setDown(false);

	QPushButton::leaveEvent(e);

	if (grabbed) {
		grabbed=false;
		releaseMouse();
	}
}

void VButton::mouseMoveEvent( QMouseEvent * e) //alex: stop repeat if outside of button
{
	if (e->buttons())
	{
		//QRect r(mapToGlobal(QPoint(0,0)),size());
		//if (!r.contains(e->globalPos())) //stop even if move inside button
		//TODO: threshold must be dpi dependent or button size dependent or fixed (15pix)?
		int threshold = qMin(size().width()/4,size().height()/4);
		if (qAbs(timerStartPos.x() - e->globalPos().x())>threshold ||
			qAbs(timerStartPos.y() - e->globalPos().y())>threshold)
		{
			stopTimer();
			setDown(false);

		}
	}
	QPushButton::mouseMoveEvent(e);
}

void VButton::repeatKey()
{
	/* dont need?
	int threshold = qMin(size().width()/4,size().height()/4);
	if (qAbs(timerStartPos.x() - QCursor::pos().x())>threshold ||
		qAbs(timerStartPos.y() - QCursor::pos().y())>threshold)
	{
		stopTimer();
		setDown(false);
		return;
	}*/

	if (!autoRepeated) {
		stopTimer();
		setDown(false);
		return;
	}

	//if the user is still pressing the button after 200 ms, we assume
	//he wants the key to be quickly repeated and we decrease the interval
	if (keyTimer->interval() == vkbdAutoRepeatDelay) {
		//TODO: make this configurable?
		keyTimer->setInterval(vkbdAutoRepeatRate);
	}
	if (rightClicked) {
		globalShift->setChecked(true);
	}
	sendKey();
}

